import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.Timer;

/**
 * A graphical dance floor for dancing turtles.
 * 
 * @author Joel Ross
 * @version Sp2014
 */
public class DanceFloor extends JPanel implements ActionListener
{
    public static final int FLOOR_WIDTH = DancingTurtle.MAX_BOUND.width+55; //dimensions of the floor, with some boundary to help
    public static final int FLOOR_HEIGHT = DancingTurtle.MAX_BOUND.height+55;

    private ArrayList<DancingTurtle> turtles; //list of dancers
    private Timer animator; //a timer to control animation

    private JFrame frame; //the containing frame
    
    /**
     * Creates a new graphical DanceFloor object for turtles to dance on! This includes a visible frame to contain the DanceFloor
     */
    public DanceFloor()
    {
        this.setPreferredSize(new Dimension(FLOOR_WIDTH, FLOOR_HEIGHT)); //set the size of the graphical frame
        turtles = new ArrayList<DancingTurtle>();

        animator = new Timer(300,this); //create the timer, with this object controlling it

        //set up the containing frame
        frame = new JFrame("Turtle Dance Party!");
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        frame.getContentPane().add(this);
        frame.pack();
        frame.setVisible(true);

    }

    /**
     * Adds a dancer to the list of dancers
     * @param turtle the dancing turtle to add
     */
    public void addDancer(DancingTurtle turtle)
    {
        turtles.add(turtle);
    }

    /**
     * Tells all of the dancers to take a step
     */
    public void stepAll()
    {
        for(DancingTurtle turtle : turtles)
        {
            turtle.step();
        }
    }

    /**
     * Overridden method for drawing the dancers
     */
    public void paintComponent(Graphics g)
    {
        g.setColor(Color.WHITE);
        g.fillRect(0, 0, FLOOR_WIDTH, FLOOR_HEIGHT);

        for(DancingTurtle turtle : turtles)
        {
            turtle.draw(g);
        }
    }

    /**
     * Starts the animation timer
     */
    public void startAnimation()
    {
        animator.start();
    }

    /**
     * Starts the animation timer with the specified delay
     * @param delay the time between animation frames (in ms)
     */
    public void startAnimation(int delay)
    {
        animator.setDelay(delay);
        startAnimation();
    }

    /**
     * Stops the animation timer
     */
    public void pauseAnimation()
    {
        animator.stop();
    }

	/**
	 * Method that controls the animation timer
	 */
	public void actionPerformed(ActionEvent e)
	{
		stepAll();
		repaint();		
	}

}
